{-
  This is the entry file which starts the whole program.
-}

import Control.Parallel(par, pseq)
-- import Control.DeepSeq(deepseq)
import System.Environment
import System.Random
import Control.Parallel.Strategies
import qualified Planet as P
import qualified Laws as L
import qualified Visualize as V


chunkSize :: Int
chunkSize = 5

depth :: Int
depth = 2000

getSeeds :: Int -> IO [Float]
getSeeds n = sequence $ replicate n $ randomRIO (0,1::Float)


genPlanets :: [Float] -> [Float] -> [P.Planet]
genPlanets []   []    = []
genPlanets [s1] [s2]  = [P.genPlanet s1 s2]
genPlanets l1   l2    = (P.genPlanet h1 h2) : genPlanets t1 t2
  where h1 = head l1
        h2 = head l2
        t1 = tail l1
        t2 = tail l2


-- Trivial approach, doesn't work.
trivial :: Int -> Float -> [P.Planet] -> [P.Planet]
trivial 0 _ ps = ps
trivial 1 i ps = [L.move p ps i| p <- ps]
trivial s i ps = trivial (s - 1) i ps'
  where ps' = [L.move p ps i| p <- ps]


-- Static partitioning
staticPart :: Int -> Float -> [P.Planet] -> [P.Planet]
staticPart 0 _ ps = ps
staticPart 1 i ps = [L.move p ps i| p <- ps]
staticPart s i ps = staticPart (s - 1) i ps'
  where ps1'    = [L.move p ps i| p <- ps1]
        ps2'    = [L.move p ps i| p <- ps2]
        ps'     = ps1' `par` ps2' `pseq` (ps1' ++ ps2')
        (ps1, ps2)  = splitAt (length ps `div` 2) ps


-- Finest granularity
finePart :: Int -> Float -> [P.Planet] -> [P.Planet]
finePart 0 _ ps = ps
finePart 1 i ps = [L.move p ps i| p <- ps] `using` parList rseq
finePart s i ps = finePart (s - 1) i ps'
  where ps'       = [L.move p ps i| p <- ps] `using` parList rseq


-- parListChunk
chunkPart :: Int -> Float -> [P.Planet] -> [P.Planet]
chunkPart 0 _ ps = ps
chunkPart 1 i ps = [L.move p ps i| p <- ps] `using` parListChunk chunkSize rdeepseq
chunkPart s i ps = chunkPart (s - 1) i ps'
  where ps' = [L.move p ps i| p <- ps] `using` parListChunk chunkSize rdeepseq


-- depth limited
depthPart :: Int -> Int -> Float -> [P.Planet] -> [P.Planet]
depthPart 0 _ _ ps  = ps
depthPart s 0 i ps  = chunkPart s i ps
-- depthPart s 0 i ps  = trivial s i ps
depthPart s d i ps  = depthPart (s - 1) (d - 1) i ps'
  where ps'         = [L.move p ps i| p <- ps] `using` parListChunk chunkSize rdeepseq


forceEval :: [P.Planet] -> IO ()
forceEval ps = do
  print $ length $ filter ((==) (P.Planet [0, 0] 0 0 0)) ps


main :: IO ()
main = do
  args <- getArgs
  let [n, s, i, mode] = args
  let planetNum = read n :: Int
  let steps     = read s :: Int
  let interval  = read i :: Float
  seedList1 <- getSeeds planetNum
  seedList2 <- getSeeds planetNum
  let planets = (genPlanets (seedList1) (seedList2)) `using` parList rseq    
  -- let planets = (genPlanets (seedList1 `using` parList rseq) (seedList2 `using` parList rseq)) `using` parList rseq
  case mode of
    "v"       -> do
      let star = P.Planet [0, 0] (1e5 * 7) 0 0
      V.runSimulation (star : planets)
    "trivial"   -> do  
      let state = (trivial steps interval planets)
      forceEval state
    "static"  -> do
      let state = (staticPart steps interval planets)
      forceEval state
    "parList" -> do
      let state = (finePart steps interval planets) `using` parList rseq
      forceEval state
    "chunk"   -> do
      let state = (chunkPart steps interval planets) `using` parList rseq
      forceEval state
    "depth"   -> do
      let state = (depthPart steps depth interval planets) `using` parList rseq
      forceEval state
    _         -> do
      print $ "Usage: ./Galaxy <Number of Bodies> <Number of Steps> <Time Inteval> <mode> +RTS -N<Number of Threads> -s"

