{-
  This module defines and exports data type Planet.
-}

module Planet (
  Planet(..),
  genPlanet
) where

import Control.DeepSeq

{-
  Units:
    velocity  m * s^-1
    mass      kg^-1
    posX      m
    poxY      m
-}
data Planet = Planet {
  velocity    :: ![Float],
  mass        :: !Float,
  posX        :: !Float,
  posY        :: !Float
} deriving (Show, Eq)

instance NFData Planet where
  rnf (Planet v m x y) = rnf v `deepseq` rnf m `deepseq` rnf x `deepseq` rnf y

{-
  Generate a planet from two seed which are randomly generated.
-}
genPlanet :: Float -> Float -> Planet
genPlanet s1 s2 = Planet [vx, vy] m x y
  where vx  = if s1 > 0.5 then (-absVx) else absVx
        vy  = -(x * vx / y)
        m   = 1e2 + d * 1e2 + s1 * 1e1
        x   = -(720 / 4) + s1 * (720 / 2)
        y   = -(720 / 4) + s2 * (720 / 2)
        -- x   = if s1 > 0.5 then 20 + s1 * 200 else -(20 + s1 * 200)
        -- y   = if s2 > 0.5 then 20 + s2 * 200 else -(20 + s2 * 200)
        absVx = 1 * 1e2 + s1 * 1e1 + (1 - d) * 1e1
        d = ((abs x) + (abs y)) / 720 / 2



