import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import csv

# parList
def timePlot(n, m, path, name):

  threads = [i for i in range(1, 9)]
  listParTimeTotal = []
  listParTimeElapsed = []
  with open(path, newline='') as listParTime:
    reader = csv.DictReader(listParTime)
    for row in reader:
      listParTimeTotal.append(float(row['total time'].strip()))
      listParTimeElapsed.append(float(row['total elapsed'].strip()))

  listParTimeSpeedUp = [listParTimeElapsed[0] / elapsed for elapsed in listParTimeElapsed]

  fig, ax = plt.subplots(figsize=(4,4), dpi=300)
  
  ax.plot(threads, listParTimeElapsed, '-ok')
  ax.set_ylim([0, max(listParTimeElapsed) + 1])
  ax.set_xlim([0, 9])
  ax.grid()
  ax.set_title('Time of ' + name + ' (n=' + str(n) + ',m=' + str(m) + ')')
  plt.savefig('./figures/' + name + 'Time.png')

# parList
def speedupPlot(n, m, path, name):

  threads = [i for i in range(1, 9)]
  listParTimeTotal = []
  listParTimeElapsed = []
  with open(path, newline='') as listParTime:
    reader = csv.DictReader(listParTime)
    for row in reader:
      listParTimeTotal.append(float(row['total time'].strip()))
      listParTimeElapsed.append(float(row['total elapsed'].strip()))

  listParTimeSpeedUp = [listParTimeElapsed[0] / elapsed for elapsed in listParTimeElapsed]

  fig, ax = plt.subplots(figsize=(4,4), dpi=300)
  
  ax.plot(threads, listParTimeSpeedUp, '-ok')
  ax.set_ylim([0, max(listParTimeSpeedUp) + 1])
  ax.set_xlim([0, 9])
  ax.grid()
  ax.set_title('Speedup of ' + name + ' (n=' + str(n) + ',m=' + str(m) + ')')
  plt.savefig('./figures/' + name + 'Speedup.png')

def main():

  timePlot(50, 10000, './stats/parList-50-10000.csv', 'parList')
  speedupPlot(50, 10000, './stats/parList-50-10000.csv', 'parList')

  timePlot(50, 10000, './stats/chunk-50-10000-size5.csv', 'chunk(5)')
  speedupPlot(50, 10000, './stats/chunk-50-10000-size5.csv', 'chunk(5)')

  timePlot(1500, 10, './stats/chunk-1500-10-size50.csv', 'chunk(50)')
  speedupPlot(1500, 10, './stats/chunk-1500-10-size50.csv', 'chunk(50)')

  timePlot(50, 10000, './stats/chunk-50-10000-8-size.csv', 'chunk(8 threads)')
  speedupPlot(50, 10000, './stats/chunk-50-10000-8-size.csv', 'chunk(8 threads)')

  # timePlot('./stats/depth-50-10000-c8-s5.csv', 'depth(8 threads, chunk size 5)')
  # speedupPlot('./stats/depth-50-10000-c8-s5.csv', 'depth(8 threads, chunk size 5)')

  timePlot(50, 10000, './stats/depth-50-10000-d1000-s5.csv', 'depth(depth 1000, chunk size 5)')
  speedupPlot(50, 10000, './stats/depth-50-10000-d1000-s5.csv', 'depth(depth 1000, chunk size 5)')

if __name__ == "__main__":
    main()